import org.jdom.CDATA;
import org.jdom.Element;
import java.util.*;
import java.awt.*;

/**
 * This class represents a layer or a subject category
 *
 * @author Alberto Simes
 */
public class Layer implements XML {
    private boolean visible;
    private Color color;
    private Vector elements;
    private String title;
    private String description;

    public Element toXML() {
	Element ge = new Element("layer");
	
	Element e_name = new Element("title");
	e_name.addContent(title);

	Element e_description = new Element("description");
	e_description.addContent(description);

	Element e_color = new Element("color");
	Element e_red = new Element("red");
	Element e_green = new Element("green");
	Element e_blue = new Element("blue");
	e_red.addContent(new Integer(color.getRed()).toString());
	e_green.addContent(new Integer(color.getGreen()).toString());
	e_blue.addContent(new Integer(color.getBlue()).toString());
	
	e_color.addContent(e_red);
	e_color.addContent(e_green);
	e_color.addContent(e_blue);

	ge.addContent(e_name);
	ge.addContent(e_description);
	ge.addContent(e_color);

	Enumeration elems = elements.elements();
	while (elems.hasMoreElements()) {
	    GeographicElement geographic_elem = (GeographicElement)elems.nextElement();
	    ge.addContent(geographic_elem.toXML());
	}
	return ge;
    }

    /**
     * The constructor initialize a layer with a title (subject name)
     *
     * @param title The title or name of the subject
     */
    public Layer(String title) {
	this.color = new Color(0, 255, 0);
	this.visible = true;
	this.title = new String(title);
	elements = new Vector();
    }

    public Layer(Element el) throws InvalidElementException {
	this.color = new Color(0, 255, 0);
	Iterator it = el.getChildren().iterator();
	elements = new Vector();
	this.visible = true;
	while(it.hasNext()) {
	    Element elem = (Element)it.next();
	    if (elem.getName().equals("title")) {
		this.title = new String(elem.getText());
	    } else 
		if (elem.getName().equals("description")) {
		    this.description = new String(elem.getText());
		}
		else
		    if (elem.getName().equals("element")) {
			elements.add(new GeographicElement(this, elem));
		    } else 
			if (elem.getName().equals("color")) {
			    int r,g,b;
			    r = Integer.parseInt(elem.getChild("red").getText());
			    g = Integer.parseInt(elem.getChild("green").getText());
			    b = Integer.parseInt(elem.getChild("blue").getText());
			    this.color = new Color(r,g,b);
			} else {
			    InvalidElementException exc = new InvalidElementException(elem.getName());
			    throw exc;
			}
	}
    }

    /**
     * Sets the description for this layer
     *
     * @param description The description for the layer
     * @return            The modified layer
     */
    public Layer setDescription(String description) {
	this.description = new String(description);
	return this;
    }


    /**
     * Adds a geographic element to the layer
     *
     * @param element The name of the element to add
     * @return        The new element created
     */
    public GeographicElement addElement(String element) {
	GeographicElement ge = new GeographicElement(this, element);
	elements.addElement(ge);
	return ge;
    }

    public void show(Graphics g, boolean z) {
	if (visible) {
	    Enumeration elems = elements.elements();
	    while(elems.hasMoreElements()) {
		GeographicElement ge = (GeographicElement)elems.nextElement();
		ge.show(g,z,this.color);
	    }
	}
    }

    /**
     * Returns a string representing the layer (only the title)
     *
     * @return the string representation for the layer
     */
    public String toString() {
	return title;
    }

    public Vector getElements() {
	return this.elements;
    }

    public boolean isVisible() {
	return this.visible;
    }

    public void setVisible(boolean v) {
	this.visible = v;
    }

    public void setColor(Color color) {
	this.color = color;
    }

    public Color getColor() {
	return this.color;
    }

}
